import requests
import os
import json

FILE_NAME = "data.txt"

def upload():
    url = 'https://api.bayfiles.com/upload'
    f = open(FILE_NAME,"w"); f.write("test"); f.close()
    f = open(FILE_NAME,"r")
    files = {'file': (FILE_NAME, f)}
    r = requests.post('https://api.bayfiles.com/upload', files=files)
    print(r.content)
    data = json.loads(r.content)
    url = data["data"]["file"]["url"]["short"]
    id = url.replace("https://bayfiles.com/", "")
    print(id)
    assert r.status_code == 200
    f.close()
    os.remove(FILE_NAME)
    return id

def download(id):
    url = f'https://bayfiles.com/{id}/{FILE_NAME}'
    r = requests.get(url)
    print(r.content)
    assert r.status_code == 200

def test_code():
    id = upload()
    download(id)

test_code()



# -*- coding: UTF-8 -*-
# todo: ipv6 and ipv4 service open together
#       tcp hole punching like bt software
#       multi-threading sharing
#       tcp keep alive

import socket
import os
import struct
from tqdm import tqdm

from kruz.settings import *

def read_file_size(file):
    size = file.seek(0, os.SEEK_END)
    file.seek(0)
    return size

def send_file(s,file):
    """
    构造发送的包头
    包头格式: |Type|File name Length|File Length
             Type: char[2]
             File name Length: char[2]
             File Length: uint
    ==============
    Type   | 含义
    ==============
    0x00 | 文件
    0x01 | 待定
    ==============
    发送完包头之后再发具体内容
    :param type: 包头的Type
    :param length: 要发送的数据长度
    :return: 包头
    """
    length = read_file_size(file)
    name = os.path.basename(file.name)
    if len(name) > 0xffff: 
        raise Exception("file name is too long")
    head = b""
    head = struct.pack('<2s2sI', 
            b'\x00\x00',
            len(name).to_bytes(2,byteorder="little"),
            length)
    s.send(head)
    s.send(name.encode("utf-8"))
    s.sendfile(file)

def run_server(filename,port,ipv6=True):
    f = open(filename,"rb")
    addr = ("", port)
    if socket.has_dualstack_ipv6() and ipv6:
        s = socket.create_server(addr, family=socket.AF_INET6, dualstack_ipv6=True)
    else:
        s = socket.create_server(addr)
        print("IPv6 is not supported")
    
    s.listen(5)
    while True:
        c, addr = s.accept()
        send_file(c,f)
        break

def run_client(ip,port):
    if socket.has_dualstack_ipv6() and ":" in ip:
        s = socket.socket(family=socket.AF_INET6)
        s.connect((ip, port, 0, 0))
    else:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((ip, port))
    
    # analyze head
    head = s.recv(8)
    head_type, filename_length, length = struct.unpack('<2s2sI', head)
    if head_type == b'\x00\x00':
        tmp = int.from_bytes(filename_length, byteorder="little")
        filename = s.recv(tmp).decode()
        f = open(filename,"wb")
        progress = tqdm(total=length,unit='B', unit_divisor=CHUNK_SIZE,unit_scale=True,desc=filename)
        for _ in range(length//CHUNK_SIZE):
            progress.update(CHUNK_SIZE)
            f.write(s.recv(CHUNK_SIZE))
        f.close()
    assert True


#!/usr/bin/python
# -*- coding: UTF-8 -*-

import argparse
import socket
import time

import stun
from kruz import file_transfer_tcp as ft
from kruz.__version__ import __version__
from kruz.settings import DEFAULTS


def show_info(src_ip,src_port,enable_ipv6=True):
    if socket.has_dualstack_ipv6() and enable_ipv6:
        ipv6 = socket.getaddrinfo(socket.gethostname(), None, socket.AF_INET6)[1][4][0]
        print("Your Computer is supported IPv6.")
        print(f'IPv6 IP:Port: {ipv6}:{src_port}')
    nat_type, external_ip, external_port = stun.get_ip_info(
        src_ip, src_port,
        stun_host=DEFAULTS['stun_ip'],
        stun_port=DEFAULTS['stun_port'])
    lan_ips = socket.gethostbyname_ex(socket.gethostname())[2]
    print('NAT Type:', nat_type)
    print(f'External IP:Port: {external_ip}:{external_port}')
    for i in range(len(lan_ips)):
        print(f'LAN {i} IP:Port: {lan_ips[i]}:{src_port}')

def show_command_tips():
    pass

def arg_parse(parser,args):
    if args.show:
        show_info(args.src_ip,args.src_port)
        print('')
        print("Note: These IP and Port might be changed dynamically.")
        parser.exit()
    elif args.dst_ip and args.dst_port:
        print(f'Recving file from {args.dst_ip}:{args.dst_port}')
        tmp = time.time()
        ft.run_client(args.dst_ip,args.dst_port)
        print(f'File recved, cost {time.time()-tmp}s')
        parser.exit()
        

def run(args):
    show_info(args.src_ip,args.src_port)
    ft.run_server(args.filename,args.src_port)

def main():
    parser = argparse.ArgumentParser(description='A command line p2p file transfer')
    parser.add_argument('-v', '--version', help='version 版本信息',action='version',version='kruz v'+__version__)
    parser.add_argument('-d', '--debug', help='debug 显示调试信息', action="store_true",default=DEFAULTS['debug'])
    parser.add_argument('-s', '--show', help='show 显示本机信息', action='store_true')
    parser.add_argument('-p', '--src_port', help='port to listen on for sender 本机端口', type=int, default=DEFAULTS['src_port'])
    parser.add_argument('-i', '--src_ip', help='network interface for sender 本机IP', default=DEFAULTS['src_ip'])
    parser.add_argument('-P', '--dst_port', type=int,help='destination port which send file 目的端口')
    parser.add_argument('-I', '--dst_ip',help='destination ip which send file 目的IP')
    parser.add_argument('filename', type=str, nargs='?',
                    help='filename 文件名')
    args = parser.parse_args()
    arg_parse(parser,args)
    if args.filename:
        run(args)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

DEFAULTS = {}
DEFAULTS['debug'] = False
DEFAULTS['src_ip'] = '0.0.0.0'
DEFAULTS['src_port'] = 19302
DEFAULTS['stun_ip'] = 'stun.l.google.com'
DEFAULTS['stun_port'] = 19302

CHUNK_SIZE = 4096